library(tidyverse)
library(eiPack)
library(rgdal)
library(foreign)
library(readr)

sts <- c('AL','AR','AZ','CA','DE','FL','GA','IL','LA','MD','MS','NC','NM','NV','NY','SC','TN','TX','VA')

library("foreach")
library("doParallel")

cl <- makeCluster(length(sts)) ; registerDoParallel(cl) 
drive <- "/mnt/shares/lsa/"

res = foreach(st = sts, 
	              .combine = "rbind", 
	              .packages = c("eiPack","rgdal","foreign","readr")) %dopar% {
		
	bc <- read.dbf(paste(drive,"EI/votes/",st,".dbf",sep='')); ## if(st=="CA"){bc$Obama<-bc$PRSDEM01; bc$Romney<-bc$PRSREP01;}
	bc$cty <- substr(bc$block,3,5)
	b <- as.data.frame(read_delim(paste(drive,"EI/race/",st,".txt",sep=''), col_names=T,delim='\t')); b$Obama<-bc$Obama[match(b$geoid,bc$block)]; b$Romney<-bc$Romney[match(b$geoid,bc$block)]; b$cty <- substr(b$geoid,3,5)
	b$Obama[is.na(b$Obama)]<-0; b$Romney[is.na(b$Romney)]<-0; b$hNBvap <- b$hTvap - (b$Bvap-b$nBvap) ##b$ <- b$Bvap-b$nBvap ## hispanic blacks

	outfile <- paste(drive,"EI/ei/",st,'.txt',sep=''); ##file.remove(outfile); 
	ctys <- unique(b$cty); res <- NULL;

	for(cty in ctys){
		bs <- b[b$cty==cty,]; bs<-bs[bs$Tvap>0,]; print(paste('cty:',cty))
		a <- aggregate(bs[,c('Tpop','Tvap','Bvap','hNBvap','Obama','Romney')],by=list(bs$VTD),FUN=sum); names(a)[names(a)=="hNBvap"] <- 'Hvap'
		a$Obama<-round(a$Obama); a$Romney<-round(a$Romney); 
		a$Ovap<-a$Tvap-a$Bvap-a$Hvap; 
		
		a$totvotes<-a$Romney+a$Obama; a$factor <- 1; a$factor[a$Tvap < a$totvotes] <- a$Tvap[a$Tvap < a$totvotes] / a$totvotes[a$Tvap < a$totvotes]
		a$Obama <- floor(a$Obama*a$factor); a$Romney <- floor(a$Romney*a$factor); ## a$notobama<- a$VAP-a$Obama; a$notromney<- a$VAP-a$Romney; 
		a$totvotes<-a$Romney+a$Obama; a$notvotes<-a$Tvap-a$totvotes

		formula1 <- cbind(totvotes,notvotes) ~ cbind(Bvap,Hvap,Ovap)
		tune.nocov <- tuneMD(formula1, data = a, ntunes = 10, totaldraws = 1000) ## change back to: tune.nocov <- tuneMD(formula1, data = a, ntunes = 10, totaldraws = 100000)
		md.out <- ei.MD.bayes(formula1, data = a, sample = 1000, thin = 100, burnin=1000, ret.mcmc=TRUE, tune.list = tune.nocov) 

		qq <-  md.out$draws$Beta
		if(dim(a)[1]>1){a$ei.Bvotrate <- apply(qq[,c(6*(1:dim(a)[1])-5)],2,mean)}else{a$ei.Bvotrate<-mean(qq[,c(6*(1:dim(a)[1])-5)])}; a$ei.Bvot <- round( a$ei.Bvotrate * a$Bvap ); a$ei.Bvot[is.na(a$ei.Bvot)] <- 0;
		if(dim(a)[1]>1){a$ei.Hvotrate <- apply(qq[,c(6*(1:dim(a)[1])-4)],2,mean)}else{a$ei.Hvotrate<-mean(qq[,c(6*(1:dim(a)[1])-4)])}; a$ei.Hvot <- round( a$ei.Hvotrate * a$Hvap ); a$ei.Hvot[is.na(a$ei.Hvot)] <- 0;
		 
		inds <- which(a$totvotes<a$ei.Bvot+a$ei.Hvot); a$ei.Bvot[inds] <- floor(a$ei.Bvot[inds]*(a$totvotes[inds]/(a$ei.Bvot[inds]+a$ei.Hvot[inds]))); a$ei.Hvot[inds] <- floor(a$ei.Hvot[inds]*(a$totvotes[inds]/(a$ei.Bvot[inds]+a$ei.Hvot[inds]))); a$ei.Bvot[is.na(a$ei.Bvot)] <- 0; a$ei.Hvot[is.na(a$ei.Hvot)] <- 0;
		if(dim(a)[1]>1){a$ei.Ovotrate <- apply(qq[,c(6*(1:dim(a)[1])-3)],2,mean)}else{a$ei.Ovotrate<-mean(qq[,c(6*(1:dim(a)[1])-3)])}; a$ei.Ovot <- a$totvotes - a$ei.Bvot - a$ei.Hvot;

		formula1 <- cbind(Romney,Obama) ~ cbind(ei.Bvot,ei.Hvot,ei.Ovot)
		tune.nocov <- tuneMD(formula1, data = a, ntunes = 10, totaldraws = 1000) ## change back to: tune.nocov <- tuneMD(formula1, data = a, ntunes = 10, totaldraws = 100000)
		md.out <- ei.MD.bayes(formula1, data = a, sample = 1000, thin = 100, burnin=1000, ret.mcmc=TRUE, tune.list = tune.nocov) 

		qq <-  md.out$draws$Beta
		if(dim(a)[1]>1){a$ei.Brom <- apply(qq[,c(6*(1:dim(a)[1])-5)],2,mean)}else{a$ei.Brom <- mean(qq[,c(6*(1:dim(a)[1])-5)])}
		if(dim(a)[1]>1){a$ei.Hrom <- apply(qq[,c(6*(1:dim(a)[1])-4)],2,mean)}else{a$ei.Hrom <- mean(qq[,c(6*(1:dim(a)[1])-4)])}
		if(dim(a)[1]>1){a$ei.Orom <- apply(qq[,c(6*(1:dim(a)[1])-3)],2,mean)}else{a$ei.Orom <- mean(qq[,c(6*(1:dim(a)[1])-3)])}
		a; sum(a$ei.Brom*a$ei.Bvot)/sum(a$ei.Bvot); sum(a$ei.Hrom*a$ei.Hvot)/sum(a$ei.Hvot); sum(a$ei.Orom*a$ei.Ovot)/sum(a$ei.Ovot)

		bs$bvotes <- bs$Bvap*a$ei.Bvotrate[match(bs$VTD,a$Group.1)]		
		bs$hvotes <- bs$hNBvap*a$ei.Hvotrate[match(bs$VTD,a$Group.1)]		
		bs$ovotes <- (bs$Tvap-bs$Bvap-bs$hNBvap)*a$ei.Ovotrate[match(bs$VTD,a$Group.1)]		
		bs$BRvotes <- bs$bvotes*a$ei.Brom[match(bs$VTD,a$Group.1)]	
		bs$BDvotes <- bs$bvotes*(1-a$ei.Brom[match(bs$VTD,a$Group.1)]	)	
		bs$HRvotes <- bs$hvotes*a$ei.Hrom[match(bs$VTD,a$Group.1)]	
		bs$HDvotes <- bs$hvotes*(1-a$ei.Hrom[match(bs$VTD,a$Group.1)]	)	
		bs$ORvotes <- bs$ovotes*a$ei.Orom[match(bs$VTD,a$Group.1)]	
		bs$ODvotes <- bs$ovotes*(1-a$ei.Orom[match(bs$VTD,a$Group.1)]	)

		write.table(bs, outfile, row.names=F,col.names=(!file.exists(outfile)), append=file.exists(outfile), sep='\t'); res<-rbind(res,bs)
	} 
} 




